@@ -3,20 +3,22 @@ package DateTime::Format::Natural;
 use strict;
 use warnings;
 use base qw(
+    DateTime::Format::Natural::Aliases
     DateTime::Format::Natural::Base
     DateTime::Format::Natural::Duration
+    DateTime::Format::Natural::Formatted
     DateTime::Format::Natural::Helpers
 );
 use boolean qw(true false);
 
 use Carp qw(croak);
 use DateTime ();
-use List::MoreUtils qw(all any);
+use List::MoreUtils qw(all any none);
 use Params::Validate ':all';
 use Scalar::Util qw(blessed);
 use Storable qw(dclone);
 
-our $VERSION = '0.82';
+our $VERSION = '0.84';
 
 validation_options(
     on_fail => sub
@@ -135,96 +137,50 @@ sub parse_datetime
     $self->{Input_string} = $self->{Date_string};
 
     my $date_string = $self->{Date_string};
+
+    $self->_rewrite_aliases(\$date_string);
     $date_string =~ tr/,//d;
 
-    my @count = $date_string =~ m![-./]!g;
-    my %count; $count{$_}++ foreach @count;
+    my ($formatted) = $date_string =~ m!^((?:\d+?[-./])+ (?:\d+?)) \b!x;
+
+    my %count;
+    if (defined $formatted) {
+        my @count = $formatted =~ m![-./]!g;
+        $count{$_}++ foreach @count;
+    }
 
     $self->{tokens} = [];
+    $self->{traces} = [];
 
     if (scalar keys %count == 1 && $count{(keys %count)[0]} == 2) {
-        if ($date_string =~ /^\S+\b \s+ \b\S+/x) {
-            ($date_string, @{$self->{tokens}}) = split /\s+/, $date_string;
-            $self->{count}{tokens} = 1 + @{$self->{tokens}};
-        }
-        else {
-            $self->{count}{tokens} = 1;
+        my $dt = $self->_parse_formatted_ymd($date_string, \%count);
+        return $dt if blessed($dt);
+    }
+    elsif (scalar keys %count == 1 && $count{(keys %count)[0]} == 1 && (keys %count)[0] eq '/') {
+        my $dt = $self->_parse_formatted_md($date_string);
+        return $dt if blessed($dt);
+
+        if ($self->{Prefer_future}) {
+            $self->_advance_future(qw(md));
         }
+    }
+    elsif ($date_string =~ /^([+-]) (\d+?) ([a-zA-Z]+)$/x) {
+        my ($prefix, $value, $unit) = ($1, $2, lc $3);
 
-        my $separator = quotemeta((keys %count)[0]);
-        my @chunks = split /$separator/, $date_string;
-
-        my $i = 0;
-        my %length = map { length $_ => $i++ } @chunks;
-
-        my $format = $self->{Format};
-
-        if (exists $length{4}) {
-            $format = join $separator,
-              ($length{4} == 0
-                ? qw(yyyy mm dd)
-                : ($format =~ /^m/
-                    ? qw(mm dd yyyy)
-                    : qw(dd mm yyyy)
-                  )
-              );
-        }
-        else {
-            $separator = do { local $_ = $format;
-                              tr/a-zA-Z//d;
-                              tr/a-zA-Z//cs;
-                              quotemeta; };
-        }
-
-        my @separated_order = split /$separator/, $format;
-        my $separated_index = 0;
-
-        my $separated_indices = { map { substr($_, 0, 1) => $separated_index++ } @separated_order };
-
-        my @bits = split /$separator/, $date_string;
-
-        my $century = $self->{datetime}
-                    ? int($self->{datetime}->year / 100)
-                    : substr((localtime)[5] + 1900, 0, 2);
-
-        my ($day, $month, $year) = map { $bits[$separated_indices->{$_}] } qw(d m y);
-
-        if (not defined $day && defined $month && defined $year) {
-            $self->_set_failure;
-            $self->_set_error("('format' parameter invalid)");
-            return $self->_get_datetime_object;
-        }
-
-        if (length $year == 2) { $year = "$century$year" };
-
-        unless ($self->_check_date($year, $month, $day)) {
-            $self->_set_failure;
-            $self->_set_error("(invalid date)");
-            return $self->_get_datetime_object;
-        }
-
-        $self->{datetime}->set(
-            year  => $year,
-            month => $month,
-            day   => $day,
+        my %methods = (
+            '+' => '_add',
+            '-' => '_subtract',
         );
-        $self->{datetime}->truncate(to => 'day');
+        my $method = $methods{$prefix};
+
+        if (none { $unit =~ /${_}s?/ } @{$self->{data}->__units('ordered')}) {
+            $self->_set_failure;
+            $self->_set_error("(invalid unit)");
+            return $self->_get_datetime_object;
+        }
+        $self->$method($unit => $value);
 
         $self->_set_valid_exp;
-
-        if (@{$self->{tokens}}) {
-            $self->{count}{tokens}--;
-            $self->_unset_valid_exp;
-            $self->_process;
-        }
-
-        if ($self->{duration}) {
-            %{$self->{formatted}} = (
-                year  => $year,
-                month => $month,
-                day   => $day,
-            );
-        }
     }
     else {
         @{$self->{tokens}} = split /\s+/, $date_string;
@@ -234,6 +190,11 @@ sub parse_datetime
         $self->_process;
     }
 
+    my $trace = $self->_trace_string;
+    if (defined $trace) {
+        @{$self->{traces}} = $trace;
+    }
+
     return $self->_get_datetime_object;
 }
 
@@ -276,15 +237,11 @@ sub _parse_init
         }
     };
 
-    unless ($self->{running_tests}) {
-        if (exists $self->{formatted}) {
-            $set_datetime->('new', {
-                map { $_ => $self->{formatted}{$_} } qw(year month day)
-            });
-        }
-        else {
-            $set_datetime->('now', {});
-        }
+    if ($self->{running_tests}) {
+        $self->{datetime} = $self->{datetime_test}->clone;
+    }
+    else {
+        $set_datetime->('now', {});
     }
 
     $self->_init_vars;
@@ -303,7 +260,7 @@ sub parse_datetime_duration
     $self->_params_init(@_, { string => \$duration_string });
     my $timespan_sep = $self->{data}->__timespan('literal');
 
-    my @date_strings = $duration_string =~ /$timespan_sep/i
+    my @date_strings = $duration_string =~ /\b $timespan_sep \b/ix
       ? do { $self->{duration} = true;
              split /\s+ $timespan_sep \s+/ix, $duration_string }
       : do { $self->{duration} = false;
@@ -312,7 +269,7 @@ sub parse_datetime_duration
     $self->_pre_duration(\@date_strings);
     $self->{state} = {};
 
-    my @queue;
+    my (@queue, @traces);
     foreach my $date_string (@date_strings) {
         push @queue, $self->parse_datetime($date_string);
         $self->_save_state(
@@ -320,6 +277,9 @@ sub parse_datetime_duration
             failure          => $self->_get_failure,
             error            => $self->_get_error,
         );
+        if (@{$self->{traces}}) {
+            push @traces, $self->{traces}[0];
+        }
     }
 
     $self->_post_duration(\@queue);
@@ -329,6 +289,7 @@ sub parse_datetime_duration
         delete $self->{$member};
     }
 
+    @{$self->{traces}} = @traces;
     $self->{Input_string} = $duration_string;
 
     return @queue;
@@ -357,9 +318,7 @@ sub trace
 {
     my $self = shift;
 
-    return join "\n", @{$self->{trace}},
-      map  { my $unit = $_; "$unit: $self->{modified}{$unit}" }
-      keys %{$self->{modified}};
+    return @{$self->{traces}};
 }
 
 sub _process
@@ -438,8 +397,8 @@ sub _process
                             ? $regex_stack{$index}
                             : ${$self->_token($index)};
                     }
-                    my $meth = 'SUPER::'.$expression->[5]->[$i];
-                    $self->$meth(@values, $expression->[4]->[$i++]);
+                    my $worker = "SUPER::$expression->[5]->[$i]";
+                    $self->$worker(@values, $expression->[4]->[$i++]);
                 }
                 %opts = %{$expression->[6]};
                 last PARSE;
@@ -462,44 +421,52 @@ sub _post_process
     if ($self->{Prefer_future} &&
         (exists $opts{prefer_future} && $opts{prefer_future})
     ) {
-        my %modified = map { $_ => true } keys %{$self->{modified}};
-        my $token_contains = sub
-        {
-            my ($identifier) = @_;
-            return any {
-              my $data = $_;
-              any {
-                my $token = $_;
-                $token =~ /$data/i;
-              } @{$self->{tokens}}
-            } @{$self->{data}->{$identifier}};
-        };
+        $self->_advance_future;
+    }
+}
 
-        if ((all { /^(?:second|minute|hour)$/ } keys %modified)
-            && (exists $self->{modified}{hour} && $self->{modified}{hour} == 1)
-            && (exists $self->{modified}{minute} && $self->{modified}{minute} == 1)
-            && $self->{datetime}->hour < DateTime->now(time_zone => $self->{Time_zone})->hour
-        ) {
-            $self->{postprocess}{day} = 1;
-        }
-        elsif ($token_contains->('weekdays_all')
-            && (exists $self->{modified}{day} && $self->{modified}{day} == 1)
-            && ($self->_Day_of_Week($self->{datetime}->year, $self->{datetime}->month, $self->{datetime}->day)
-             < DateTime->now(time_zone => $self->{Time_zone})->wday)
-        ) {
-            $self->{postprocess}{day} = 7;
-        }
-        elsif ($token_contains->('months_all')
-            && (all { /^(?:day|month)$/ } keys %modified)
-            && (exists $self->{modified}{month} && $self->{modified}{month} == 1)
-            && (exists $self->{modified}{day}
-                  ? $self->{modified}{day} == 1
-                    ? true : false
-                  : true)
-            && ($self->{datetime}->day_of_year < DateTime->now->day_of_year)
-        ) {
-            $self->{postprocess}{year} = 1;
-        }
+sub _advance_future
+{
+    my $self = shift;
+    my %advance = map { $_ => true } @_;
+
+    my %modified = map { $_ => true } keys %{$self->{modified}};
+    my $token_contains = sub
+    {
+        my ($identifier) = @_;
+        return any {
+          my $data = $_;
+          any {
+            my $token = $_;
+            $token =~ /$data/i;
+          } @{$self->{tokens}}
+        } @{$self->{data}->{$identifier}};
+    };
+
+    if ((all { /^(?:second|minute|hour)$/ } keys %modified)
+        && (exists $self->{modified}{hour} && $self->{modified}{hour} == 1)
+        && (exists $self->{modified}{minute} && $self->{modified}{minute} == 1)
+        && $self->{datetime}->hour < DateTime->now(time_zone => $self->{Time_zone})->hour
+    ) {
+        $self->{postprocess}{day} = 1;
+    }
+    elsif ($token_contains->('weekdays_all')
+        && (exists $self->{modified}{day} && $self->{modified}{day} == 1)
+        && ($self->_Day_of_Week($self->{datetime}->year, $self->{datetime}->month, $self->{datetime}->day)
+         < DateTime->now(time_zone => $self->{Time_zone})->wday)
+    ) {
+        $self->{postprocess}{day} = 7;
+    }
+    elsif (($token_contains->('months_all') || $advance{md})
+        && (all { /^(?:day|month)$/ } keys %modified)
+        && (exists $self->{modified}{month} && $self->{modified}{month} == 1)
+        && (exists $self->{modified}{day}
+              ? $self->{modified}{day} == 1
+                ? true : false
+              : true)
+        && ($self->{datetime}->day_of_year < DateTime->now->day_of_year)
+    ) {
+        $self->{postprocess}{year} = 1;
     }
 }
 
@@ -536,7 +503,7 @@ sub _get_datetime_object
     my $self = shift;
 
     my $dt = DateTime->new(
-        time_zone => $self->{datetime}->time_zone->name,
+        time_zone => $self->{datetime}->time_zone,
         year      => $self->{datetime}->year,
         month     => $self->{datetime}->month,
         day       => $self->{datetime}->day_of_month,
@@ -558,7 +525,7 @@ sub _set_datetime
     my $self = shift;
     my ($time, $tz) = @_;
 
-    $self->{datetime} = DateTime->new(
+    $self->{datetime_test} = DateTime->new(
         time_zone => $tz || 'floating',
         %$time,
     );
@@ -593,6 +560,8 @@ DateTime::Format::Natural - Create machine readable date/time with natural parsi
      warn $parser->error;
  }
 
+ @traces = $parser->trace;
+
 =head1 DESCRIPTION
 
 C<DateTime::Format::Natural> takes a string with a human readable date/time and creates a
@@ -687,8 +656,9 @@ Returns the error message if the parsing did not succeed.
 
 =head2 trace
 
-Returns a trace of methods which were called within the Base class and
-a summary how often certain units have been modified.
+Returns one or more strings with traces of methods which were called within
+the Base class and a summary how often certain units have been modified.
+More than one string is commonly returned for durations.
 
 =head1 GRAMMAR
 
@@ -703,6 +673,18 @@ you're intending to hack a bit on the grammar guts.
 See the classes C<DateTime::Format::Natural::Lang::[language_code]> for a
 overview of currently valid input.
 
+=head1 BUGS & CAVEATS
+
+C<parse_datetime()>/C<parse_datetime_duration()> always return one or more
+DateTime objects regardless whether the parse was successful or not. In
+case no valid expression was found or a failure occurred, an unaltered
+DateTime object with its initial values (most often the "current" now) is
+likely to be returned. It is therefore recommended to use C<success()> to
+assert that the parse did succeed (at least, for common uses), otherwise
+the absence of a parse failure cannot be guaranteed.
+
+C<parse_datetime()> is not capable of handling durations.
+
 =head1 CREDITS
 
 Thanks to Tatsuhiko Miyagawa for the initial inspiration. See Miyagawa's journal
@@ -734,6 +716,8 @@ valuable suggestions and patches:
  Christian Brink
  Giovanni Pensa
  Andrew Sterling Hanenkamp
+ Eric Wilhelm
+ Kevin Field
 
 =head1 SEE ALSO
 
