@@ -5,7 +5,7 @@ use warnings;
 
 our ($VERSION, $AUTOLOAD);
 
-$VERSION = '0.9';
+$VERSION = '1.04';
 
 sub __new
 {
@@ -14,12 +14,16 @@ sub __new
     no strict 'refs';
 
     my $obj = {};
-    $obj->{weekdays}        = \%{$class.'::'.'data_weekdays'};
-    $obj->{weekdays_abbrev} = \%{$class.'::'.'data_weekdays_abbrev'};
-    $obj->{weekdays_all}    = \@{$class.'::'.'data_weekdays_all'};
-    $obj->{months}          = \%{$class.'::'.'data_months'};
-    $obj->{months_abbrev}   = \%{$class.'::'.'data_months_abbrev'};
-    $obj->{months_all}      = \@{$class.'::'.'data_months_all'};
+    $obj->{weekdays}        = \%{"${class}::data_weekdays"};
+    $obj->{weekdays_abbrev} = \%{"${class}::data_weekdays_abbrev"};
+    $obj->{weekdays_all}    = \@{"${class}::data_weekdays_all"};
+    $obj->{months}          = \%{"${class}::data_months"};
+    $obj->{months_abbrev}   = \%{"${class}::data_months_abbrev"};
+    $obj->{months_all}      = \@{"${class}::data_months_all"};
+    $obj->{conversion}      = \%{"${class}::data_conversion"};
+    $obj->{helpers}         = \%{"${class}::data_helpers"};
+    $obj->{duration}        = \%{"${class}::data_duration"};
+    $obj->{aliases}         = \%{"${class}::data_aliases"};
 
     return bless $obj, ref($class) || $class;
 }
@@ -73,6 +77,6 @@ Steven Schubiger <schubiger@cpan.org>
 This program is free software; you may redistribute it and/or
 modify it under the same terms as Perl itself.
 
-See L<http://www.perl.com/perl/misc/Artistic.html>
+See L<http://dev.perl.org/licenses/>
 
 =cut
